var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { EVMContractAddress, ERC20ContractError, } from "@snickerdoodlelabs/objects";
import { injectable } from "inversify";
import { ResultAsync } from "neverthrow";
import { BaseContract } from "../implementations/BaseContract.js";
import { ERewardRoles } from "../interfaces/enums/index.js";
import { ContractsAbis } from "../interfaces/objects/index.js";
let ERC20RewardContract = class ERC20RewardContract extends BaseContract {
    constructor(providerOrSigner, contractAddress) {
        super(providerOrSigner, contractAddress, ContractsAbis.ERC20Reward.abi);
        this.providerOrSigner = providerOrSigner;
        this.contractAddress = contractAddress;
        this.filters = {
            Transfer: (fromAddress, toAddress) => {
                return this.contract.filters.Transfer(fromAddress, toAddress);
            },
        };
    }
    name() {
        return ResultAsync.fromPromise(this.contract.name(), (e) => {
            return this.generateError(e, "Unable to call name()");
        });
    }
    symbol() {
        return ResultAsync.fromPromise(this.contract.symbol(), (e) => {
            return this.generateError(e, "Unable to call symbol()");
        });
    }
    decimals() {
        return ResultAsync.fromPromise(this.contract.decimals(), (e) => {
            return this.generateError(e, "Unable to call decimals()");
        }).map((decimalsBN) => {
            return Number(decimalsBN);
        });
    }
    totalSupply() {
        return ResultAsync.fromPromise(this.contract.totalSupply(), (e) => {
            return this.generateError(e, "Unable to call totalSupply()");
        });
    }
    balanceOf(address) {
        return ResultAsync.fromPromise(this.contract.balanceOf(address), (e) => {
            return this.generateError(e, "Unable to call balanceOf()");
        });
    }
    allowance(owner, spender) {
        return ResultAsync.fromPromise(this.contract.allowance(owner, spender), (e) => {
            return this.generateError(e, "Unable to call allowance()");
        });
    }
    approve(spender, amount, overrides) {
        return this.writeToContract("approve", [spender, amount, overrides]);
    }
    transfer(recipient, amount, overrides) {
        return this.writeToContract("transfer", [recipient, amount, overrides]);
    }
    transferFrom(sender, recipient, amount, overrides) {
        return this.writeToContract("transferFrom", [
            sender,
            recipient,
            amount,
            overrides,
        ]);
    }
    grantRole(role, address, overrides) {
        return this.writeToContract("grantRole", [ERewardRoles[role], address], overrides);
    }
    revokeRole(role, address, overrides) {
        return this.writeToContract("revokeRole", [ERewardRoles[role], address], overrides);
    }
    renounceRole(role, address, overrides) {
        return this.writeToContract("renounceRole", [ERewardRoles[role], address], overrides);
    }
    addDomain(domain, overrides) {
        return this.writeToContract("addDomain", [domain], overrides);
    }
    removeDomain(domain, overrides) {
        return this.writeToContract("removeDomain", [domain], overrides);
    }
    getDomains() {
        return ResultAsync.fromPromise(
        // returns array of domains
        this.contract.getDomains(), (e) => {
            return this.generateError(e, "Unable to call getDomains()");
        });
    }
    generateContractSpecificError(msg, e, transaction) {
        return new ERC20ContractError(msg, e, transaction);
    }
};
ERC20RewardContract = __decorate([
    injectable(),
    __metadata("design:paramtypes", [Object, String])
], ERC20RewardContract);
export { ERC20RewardContract };
//# sourceMappingURL=ERC20RewardContract.js.map