var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var ZkSyncRewardsContractFactory_1;
import { EVMContractAddress, EVMAccountAddress, RewardsFactoryError, ECreatedRewardType, } from "@snickerdoodlelabs/objects";
import { ethers } from "ethers";
import { injectable } from "inversify";
import { ResultAsync } from "neverthrow";
import { ContractFactory } from "zksync-ethers";
import { BaseContract } from "../../implementations/BaseContract.js";
import { GasUtils } from "../../implementations/GasUtils.js";
import { ContractsAbis, } from "../../interfaces/objects/index.js";
let ZkSyncRewardsContractFactory = ZkSyncRewardsContractFactory_1 = class ZkSyncRewardsContractFactory extends BaseContract {
    constructor(providerOrSigner, // needs to use ZkSync provider and wallet
    rewardType) {
        super(providerOrSigner, EVMContractAddress(ethers.ZeroAddress), // The rewards contract factory deploys a new contract, hence doesn't have a contract address
        ContractsAbis.ZkSyncERC721RewardAbi.abi);
        this.providerOrSigner = providerOrSigner;
        this.rewardType = rewardType;
        // Set the correct contract factory based on rewardTypeToDeploy
        this.contractFactory = new ContractFactory(ContractsAbis.ZkSyncERC721RewardAbi.abi, ContractsAbis.ZkSyncERC721RewardAbi.bytecode, providerOrSigner);
        this.rewardTypeToDeploy = rewardType;
    }
    // function to deploy a new ERC721 reward contract on ZkSyncEra
    deployERC721Reward(name, symbol, baseURI, overrides) {
        return GasUtils.getGasFee(this.providerOrSigner).andThen((gasFee) => {
            const contractOverrides = {
                ...gasFee,
                ...overrides,
            };
            return this.writeToContractFactory("deploy", [name, symbol, baseURI], contractOverrides, true);
        });
    }
    estimateGasToDeployERC721Contract(name, symbol, baseURI) {
        return ResultAsync.fromPromise(this.contractFactory.getDeployTransaction(name, symbol, baseURI), (e) => {
            return this.generateError(e, "Unable to get deploy transaction for contract deployment for ERC721 contract on ZKSyncRewardsContractFactory");
        })
            .andThen((deployTransaction) => {
            return ResultAsync.fromPromise(this.providerOrSigner.estimateGas(deployTransaction), (e) => {
                return this.generateError(e, "Failed to wait() for contract deployment");
            });
        })
            .map((estimatedGas) => {
            // Increase estimated gas buffer by 20%
            return (estimatedGas * 120n) / 100n;
        });
    }
    // function to deploy a new ERC721 reward contract on ZkSyncEra
    deployERC20Reward(name, symbol, overrides) {
        return GasUtils.getGasFee(this.providerOrSigner).andThen((gasFee) => {
            const contractOverrides = {
                ...gasFee,
                ...overrides,
            };
            return this.writeToContractFactory("deploy", [name, symbol], contractOverrides, true);
        });
    }
    estimateGasToDeployERC20Contract(name, symbol) {
        return ResultAsync.fromPromise(this.contractFactory.getDeployTransaction(name, symbol), (e) => {
            return this.generateError(e, "Unable to get deploy transaction for contract deployment for ERC20 contract");
        })
            .andThen((deployTransaction) => {
            return ResultAsync.fromPromise(this.providerOrSigner.estimateGas(deployTransaction), (e) => {
                return this.generateError(e, "Attempting to estimate gas for contract deployment");
            });
        })
            .map((estimatedGas) => {
            // Increase estimated gas buffer by 20%
            return (estimatedGas * 120n) / 100n;
        });
    }
    generateContractSpecificError(msg, e, transaction) {
        return new RewardsFactoryError(msg, e, transaction);
    }
    // Takes the factory's deploy function name and params, submits the transaction and returns a WrappedTransactionResponse
    writeToContractFactory(functionName, functionParams, overrides, isDeployingContract) {
        return ResultAsync.fromPromise(this.contractFactory[functionName](...functionParams, {
            ...overrides,
        }), (e) => {
            return this.generateError(e, `Unable to call ${functionName}()`);
        }).map((transactionResponse) => {
            // If we are deploying a contract, the deploy() call returns an ethers.Contract object and the txresponse is under the deployTransaction property
            return ZkSyncRewardsContractFactory_1.buildWrappedTransactionResponse(isDeployingContract == true
                ? transactionResponse.deploymentTransaction()
                : transactionResponse, EVMContractAddress(""), EVMAccountAddress(this.providerOrSigner?.address), functionName, functionParams, ContractsAbis.ZkSyncERC721RewardAbi.abi);
        });
    }
};
ZkSyncRewardsContractFactory = ZkSyncRewardsContractFactory_1 = __decorate([
    injectable(),
    __metadata("design:paramtypes", [Object, String])
], ZkSyncRewardsContractFactory);
export { ZkSyncRewardsContractFactory };
//# sourceMappingURL=ZkSyncRewardsContractFactory.js.map