import { IContentFactoryContract } from "../interfaces/IContentFactoryContract.js";
import { BaseURI, BigNumberString, ConsentFactoryContractError, ConsentName, EVMAccountAddress, EVMContractAddress, MarketplaceListing, MarketplaceTag, BlockchainCommonErrors, TransactionResponseError } from "@snickerdoodlelabs/objects";
import { ResultAsync } from "neverthrow";
import { IBaseContract } from "../interfaces/IBaseContract.js";
import { IERC7529Contract } from "../interfaces/IERC7529Contract.js";
import { WrappedTransactionResponse, ContractOverrides } from "../interfaces/objects/index.js";
export interface IConsentFactoryContract extends IBaseContract, IERC7529Contract<ConsentFactoryContractError>, IContentFactoryContract<ConsentFactoryContractError> {
    /**
     * Creates a consent contract for user
     * @param ownerAddress Address of the owner of the Consent contract instance
     * @param baseUri Base uri for the for the Consent contract instance
     * @param name Name for the for the Consent contract instance
     * @param overrides Any transaction call overrides
     */
    createConsent(ownerAddress: EVMAccountAddress, baseUri: BaseURI, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ConsentFactoryContractError>;
    setListingDuration(listingDuration: number, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ConsentFactoryContractError>;
    setMaxTagsPerListing(maxTagsPerListing: number, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ConsentFactoryContractError>;
    registerStakingToken(stakingToken: EVMContractAddress, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ConsentFactoryContractError>;
    adminRemoveListings(tag: MarketplaceTag, stakingToken: EVMContractAddress, removedSlot: BigNumberString[], overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ConsentFactoryContractError>;
    blockContentObject(stakingToken: EVMContractAddress, contentAddress: EVMContractAddress, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ConsentFactoryContractError>;
    unblockContentObject(stakingToken: EVMContractAddress, contentAddress: EVMContractAddress, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ConsentFactoryContractError>;
    /**
     *  Return the amount of gas required to create a Consent contract
     * @param ownerAddress Address of the user
     * @param baseUri URI for consent contract
     * @param name Name of the consent contract
     */
    estimateGasToCreateConsent(ownerAddress: EVMAccountAddress, baseUri: BaseURI, name: ConsentName): ResultAsync<bigint, ConsentFactoryContractError | BlockchainCommonErrors>;
    /**
     *  Return Consent addresses by checking ContractDeployed event logs
     */
    getDeployedConsents(): ResultAsync<EVMContractAddress[], ConsentFactoryContractError | BlockchainCommonErrors>;
    getListingsByTag(tag: MarketplaceTag, stakingToken: EVMContractAddress, removeExpired: boolean): ResultAsync<MarketplaceListing[], ConsentFactoryContractError | BlockchainCommonErrors>;
    getAddressOfConsentCreated(txRes: WrappedTransactionResponse): ResultAsync<EVMContractAddress, TransactionResponseError>;
}
export declare const IConsentFactoryContractType: unique symbol;
//# sourceMappingURL=IConsentFactoryContract.d.ts.map